%%%%% main script for demixing algorithm DHT)
%*Author: Mohammadreza Soltani
% Copyright (C) Mohammadreza Soltani, Chinmay Hegde 
%%% Please consider to cite the follwoing papers:
% 1- M. Soltani and C. Hegde, A Fast Iterative Algorithm for Demixing Sparse Signals from Nonlinear Observations, 
%    IEEE GlobalSIP Symposium on Compressed Sensing and Deep Learning, IEEE GlobalSIP Symposium on Compressed
%    Sensing and Deep Learning, December 2016.

% 2- M. Soltani and C. Hegde. Fast algorithms for demixing sparse signals from nonlinear observations,
%    IEEE Transactions on Signal Processing, vol. 65, no. 16, p4209-4222, August 2017.

% Date: June 2016
% Iowa State University --- Electrical and Computer Engineering*****
%%%%######################################################################

% link function
switch opts.link
    case 'sigmoid'
        g = @(x) 2./(1 + exp(-x)) - 1;
    case 'lin'
        g = @(x) x;
    case 'linsin'
        g = @(x) 2*x + sin(x);
end
Norm_err_DHT = zeros(length(svec),length(mvec),E);
for ee=1:E
    for ss = 1:length(svec)
        
        % generate signal
        w = 0*(1:n)'; z = 0*w;
        suppw = randperm(n); suppw = suppw(1:svec(ss));
        w(suppw) = sign(rand(svec(ss),1)-.5); w = w/norm(w);
        suppz = randperm(n); suppz = suppz(1:svec(ss));
        z(suppz) = sign(rand(svec(ss),1)-.5); z = z/norm(z);
        
        % basis operators
        switch opts.basis
            case 'id_dct'
                Phi = @(w) w; PhiT = @(w) w;
                Psi = @(z) dct(z); PsiT = @(z) idct(z);
            case 'haar_noiselet'
                Phi = @(w) haar(w,'forward');
                PhiT = @(w) haar(w, 'inverse');
                Psi = @(z) 1/sqrt(length(z))*realnoiselet(z);
                PsiT = @(z) 1/sqrt(length(z))*realnoiselet(z);
        end
        
        x = Phi(w) + Psi(z);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        for mm=1:length(mvec)
            % measurement operator
            switch opts.meas
                case 'gauss'
                    Amat = randn(mvec(mm),n);
                    A = @(x) Amat*x; AT = @(x) Amat'*x;
                case 'fourier'
                    zeta = sign(randn(n,1));
                    p = randperm(n)';
                    q = randperm(n/2-1)+1; % this keeps a random set of FFT coefficients
                    omega = q(1:mvec(mm)/2)';
                    A = @(z) A_f(zeta.*z, omega, p);
                    AT = @(z) zeta.*At_f(z, n, omega, p);                  
            end
            
            %%%% measurements
            y = g(A(x)) + nvara*randn(mvec(mm),1);
        
            %%%% Recovery algorithm
            recopts.s1 = svec(ss);
            recopts.s2 = svec(ss);
            recopts.m = mvec(mm);
            recopts.n = n;

            [xhatHT,w_hatm,z_hatm,xinit,w_init,z_init,err_trace] = htdemix(y,g,A,AT,Phi,PhiT,Psi,PsiT,recopts);
            Norm_err_DHT(ss,mm,ee) = norm(xhatHT-x)/norm(x);
            %***********************************************************************
            
        end
    end
end

figure,
plot(mvec,Norm_err_DHT(1,:,1),'r','LineWidth',2);
grid on
axisfortex(opts.link,'m','Norm error')
